﻿using System.Threading;
using RocketAd;
using RocketGM;
using RocketLog;
using UnityEngine;

namespace RocketRevenueTracker
{
    public class RocketRevenueTrackerController : MonoBehaviour
    {
        [SerializeField]
        private RocketRevenueTracker[] _revenueTrackers = new RocketRevenueTracker[]
        {
            new RocketRevenueTracker(0.03f, "_03", true),
            new RocketRevenueTracker(0.05f, "_05", true),
            new RocketRevenueTracker(0.10f, "_10", true),
            new RocketRevenueTracker(0.20f, "_20", true),
            new RocketRevenueTracker(0.50f, "_50", true),
            new RocketRevenueTracker(1.00f, "_100", true)
        };
        
#if ROC_GM_APPSFLYER
        [SerializeField]
        private bool _sendAppsflyer = true;
#endif
#if ROC_FACEBOOK
        [SerializeField]
        private bool _sendFacebook = true;
#endif
#if ROC_FIREBASE
[SerializeField]
        private bool _sendFirebase = true;
#endif

        private static int _unityThreadId = -1;

        private RocLog Log = new RocLog(nameof(RocketRevenueTrackerController), DebugLevels.Debug);

        private void Start()
        {
            DontDestroyOnLoad(this);
            
            _unityThreadId = Thread.CurrentThread.ManagedThreadId;
            Log.Debug($"Initializing RocGMRevenueTracker.\nThread ID: {_unityThreadId}");
            
            Log.Debug($"{_revenueTrackers.Length} tracker found.");

#if ROC_GM_APPSFLYER
            RocketRevenueTracker.SendAppsflyer = _sendAppsflyer;
#endif
#if ROC_FACEBOOK
            RocketRevenueTracker.SendFacebook = _sendFacebook;
#endif
#if ROC_FIREBASE
            RocketRevenueTracker.SendFirebase = _sendFirebase;
#endif
            
            RocGm.DoAfterInitialized(() =>
            {
                foreach (var revenueTracker in _revenueTrackers)
                {
                    revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
                }
            });
        }

        private void OnEnable()
        {
            RocAd.ImpressionTracked += OnImpressionTracked;
        }

        private void OnDisable()
        {
            RocAd.ImpressionTracked -= OnImpressionTracked;
        }

        private void OnImpressionTracked(RocImpressionData rocImpressionData)
        {
            int currentThreadId = Thread.CurrentThread.ManagedThreadId;
            Log.Debug($"Impression tracked.\nThread ID: {currentThreadId}");

            if (currentThreadId != _unityThreadId)
            {
                Log.Warning($"We are not on the Unity Thread.\nUnity Thread ID: {_unityThreadId}");
                return;
            }
            
            foreach (var revenueTracker in _revenueTrackers)
            {
                revenueTracker.RevenueAmount += (float)rocImpressionData.Revenue;
                revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
            }
        }
    }
}